#  Copyright 2024 Palantir Technologies, Inc.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""
    Palantir OpenAPI

    The Palantir REST API. Please see https://www.palantir.com/docs for more details.

    The version of the OpenAPI document: 1.738.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import io
import warnings

from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Dict, List, Optional, Tuple, Union, Any

try:
    from typing import Annotated
except ImportError:
    from typing_extensions import Annotated

from pydantic import Field
from typing_extensions import Annotated
from pydantic import StrictInt, StrictStr

from typing import Optional

from foundry.models.action_type_v2 import ActionTypeV2
from foundry.models.deployment_metadata import DeploymentMetadata
from foundry.models.link_type_side_v2 import LinkTypeSideV2
from foundry.models.list_action_types_response_v2 import ListActionTypesResponseV2
from foundry.models.list_deployments_response import ListDeploymentsResponse
from foundry.models.list_object_types_v2_response import ListObjectTypesV2Response
from foundry.models.list_ontologies_v2_response import ListOntologiesV2Response
from foundry.models.list_outgoing_link_types_response_v2 import ListOutgoingLinkTypesResponseV2
from foundry.models.list_query_types_response_v2 import ListQueryTypesResponseV2
from foundry.models.object_type_v2 import ObjectTypeV2
from foundry.models.ontology_full_metadata import OntologyFullMetadata
from foundry.models.ontology_v2 import OntologyV2
from foundry.models.query_type_v2 import QueryTypeV2

from foundry.api_client import ApiClient
from foundry._errors.sdk_internal_error import handle_unexpected


class OntologiesV2ApiServiceApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client: ApiClient) -> None:
        self._api_client = api_client

    @validate_call
    @handle_unexpected
    def get_action_type(
        self,
        ontology: Annotated[
            StrictStr,
            Field(
                description="The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**. "
            ),
        ],
        action_type: Annotated[
            StrictStr, Field(description="The name of the action type in the API. ")
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]],
        ] = None,
    ) -> ActionTypeV2:
        """get_action_type

        Gets a specific action type with the given API name.  Third-party applications using this endpoint via OAuth2 must request the following operation scope: `api:read-data`.

        :param ontology: The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**.  (required)
        :type ontology: str
        :param action_type: The name of the action type in the API.  (required)
        :type action_type: str
        :param _request_timeout: timeout setting for this request in seconds. If
                                 one number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :return: Returns the result object.
        """  # noqa: E501

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, Any]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Any = None

        # process the path parameters
        if ontology is not None:
            _path_params["ontology"] = ontology
        if action_type is not None:
            _path_params["actionType"] = action_type
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        _header_params["Accept"] = self._api_client.select_header_accept(["application/json"])

        # authentication setting
        _auth_settings: List[str] = ["BearerAuth"]

        _response_types_map: Dict[str, Optional[str]] = {"200": "ActionTypeV2"}
        return self._api_client.call_api(
            method="GET",
            resource_path="/v2/ontologies/{ontology}/actionTypes/{actionType}",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            response_types_map=_response_types_map,
        )

    @validate_call
    @handle_unexpected
    def get_deployment(
        self,
        ontology: Annotated[
            StrictStr,
            Field(
                description="The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**. "
            ),
        ],
        deployment: Annotated[
            StrictStr,
            Field(
                description="The API name of the deployment you want to fetch information about. "
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]],
        ] = None,
    ) -> DeploymentMetadata:
        """get_deployment

        Fetches information about a model deployment within a given Ontology.

        :param ontology: The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**.  (required)
        :type ontology: str
        :param deployment: The API name of the deployment you want to fetch information about.  (required)
        :type deployment: str
        :param _request_timeout: timeout setting for this request in seconds. If
                                 one number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :return: Returns the result object.
        """  # noqa: E501

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, Any]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Any = None

        # process the path parameters
        if ontology is not None:
            _path_params["ontology"] = ontology
        if deployment is not None:
            _path_params["deployment"] = deployment
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        _header_params["Accept"] = self._api_client.select_header_accept(["application/json"])

        # authentication setting
        _auth_settings: List[str] = ["BearerAuth"]

        _response_types_map: Dict[str, Optional[str]] = {"200": "DeploymentMetadata"}
        return self._api_client.call_api(
            method="GET",
            resource_path="/v2/ontologies/{ontology}/models/deployments/{deployment}",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            response_types_map=_response_types_map,
        )

    @validate_call
    @handle_unexpected
    def get_object_type(
        self,
        ontology: Annotated[
            StrictStr,
            Field(
                description="The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**. "
            ),
        ],
        object_type: Annotated[
            StrictStr,
            Field(
                description="The API name of the object type. To find the API name, use the **List object types** endpoint or check the **Ontology Manager**. "
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]],
        ] = None,
    ) -> ObjectTypeV2:
        """get_object_type

        Gets a specific object type with the given API name.  Third-party applications using this endpoint via OAuth2 must request the following operation scope: `api:read-data`.

        :param ontology: The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**.  (required)
        :type ontology: str
        :param object_type: The API name of the object type. To find the API name, use the **List object types** endpoint or check the **Ontology Manager**.  (required)
        :type object_type: str
        :param _request_timeout: timeout setting for this request in seconds. If
                                 one number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :return: Returns the result object.
        """  # noqa: E501

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, Any]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Any = None

        # process the path parameters
        if ontology is not None:
            _path_params["ontology"] = ontology
        if object_type is not None:
            _path_params["objectType"] = object_type
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        _header_params["Accept"] = self._api_client.select_header_accept(["application/json"])

        # authentication setting
        _auth_settings: List[str] = ["BearerAuth"]

        _response_types_map: Dict[str, Optional[str]] = {"200": "ObjectTypeV2"}
        return self._api_client.call_api(
            method="GET",
            resource_path="/v2/ontologies/{ontology}/objectTypes/{objectType}",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            response_types_map=_response_types_map,
        )

    @validate_call
    @handle_unexpected
    def get_ontology_full_metadata(
        self,
        ontology: Annotated[
            StrictStr,
            Field(
                description="The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**. "
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]],
        ] = None,
    ) -> OntologyFullMetadata:
        """get_ontology_full_metadata

        Get the full Ontology metadata. This includes the objects, links, actions, and queries.

        :param ontology: The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**.  (required)
        :type ontology: str
        :param _request_timeout: timeout setting for this request in seconds. If
                                 one number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :return: Returns the result object.
        """  # noqa: E501

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, Any]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Any = None

        # process the path parameters
        if ontology is not None:
            _path_params["ontology"] = ontology
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        _header_params["Accept"] = self._api_client.select_header_accept(["application/json"])

        # authentication setting
        _auth_settings: List[str] = ["BearerAuth"]

        _response_types_map: Dict[str, Optional[str]] = {"200": "OntologyFullMetadata"}
        return self._api_client.call_api(
            method="GET",
            resource_path="/v2/ontologies/{ontology}/fullMetadata",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            response_types_map=_response_types_map,
        )

    @validate_call
    @handle_unexpected
    def get_ontology(
        self,
        ontology: Annotated[
            StrictStr,
            Field(
                description="The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**. "
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]],
        ] = None,
    ) -> OntologyV2:
        """get_ontology

        Gets a specific ontology with the given Ontology RID.  Third-party applications using this endpoint via OAuth2 must request the following operation scope: `api:read-data`.

        :param ontology: The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**.  (required)
        :type ontology: str
        :param _request_timeout: timeout setting for this request in seconds. If
                                 one number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :return: Returns the result object.
        """  # noqa: E501

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, Any]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Any = None

        # process the path parameters
        if ontology is not None:
            _path_params["ontology"] = ontology
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        _header_params["Accept"] = self._api_client.select_header_accept(["application/json"])

        # authentication setting
        _auth_settings: List[str] = ["BearerAuth"]

        _response_types_map: Dict[str, Optional[str]] = {"200": "OntologyV2"}
        return self._api_client.call_api(
            method="GET",
            resource_path="/v2/ontologies/{ontology}",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            response_types_map=_response_types_map,
        )

    @validate_call
    @handle_unexpected
    def get_outgoing_link_type(
        self,
        ontology: Annotated[
            StrictStr,
            Field(
                description="The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**. "
            ),
        ],
        object_type: Annotated[
            StrictStr,
            Field(
                description="The API name of the object type. To find the API name, use the **List object types** endpoint or check the **Ontology Manager** application. "
            ),
        ],
        link_type: Annotated[
            StrictStr,
            Field(
                description="The API name of the outgoing link. To find the API name for your link type, check the **Ontology Manager**. "
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]],
        ] = None,
    ) -> LinkTypeSideV2:
        """get_outgoing_link_type

        Get an outgoing link for an object type.  Third-party applications using this endpoint via OAuth2 must request the following operation scopes: `api:read-data`.

        :param ontology: The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**.  (required)
        :type ontology: str
        :param object_type: The API name of the object type. To find the API name, use the **List object types** endpoint or check the **Ontology Manager** application.  (required)
        :type object_type: str
        :param link_type: The API name of the outgoing link. To find the API name for your link type, check the **Ontology Manager**.  (required)
        :type link_type: str
        :param _request_timeout: timeout setting for this request in seconds. If
                                 one number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :return: Returns the result object.
        """  # noqa: E501

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, Any]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Any = None

        # process the path parameters
        if ontology is not None:
            _path_params["ontology"] = ontology
        if object_type is not None:
            _path_params["objectType"] = object_type
        if link_type is not None:
            _path_params["linkType"] = link_type
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        _header_params["Accept"] = self._api_client.select_header_accept(["application/json"])

        # authentication setting
        _auth_settings: List[str] = ["BearerAuth"]

        _response_types_map: Dict[str, Optional[str]] = {"200": "LinkTypeSideV2"}
        return self._api_client.call_api(
            method="GET",
            resource_path="/v2/ontologies/{ontology}/objectTypes/{objectType}/outgoingLinkTypes/{linkType}",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            response_types_map=_response_types_map,
        )

    @validate_call
    @handle_unexpected
    def get_query_type(
        self,
        ontology: Annotated[
            StrictStr,
            Field(
                description="The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**. "
            ),
        ],
        query_api_name: Annotated[
            StrictStr,
            Field(
                description="The API name of the query type. To find the API name, use the **List query types** endpoint or check the **Ontology Manager**. "
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]],
        ] = None,
    ) -> QueryTypeV2:
        """get_query_type

        Gets a specific query type with the given API name.  Third-party applications using this endpoint via OAuth2 must request the following operation scope: `api:read-data`.

        :param ontology: The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**.  (required)
        :type ontology: str
        :param query_api_name: The API name of the query type. To find the API name, use the **List query types** endpoint or check the **Ontology Manager**.  (required)
        :type query_api_name: str
        :param _request_timeout: timeout setting for this request in seconds. If
                                 one number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :return: Returns the result object.
        """  # noqa: E501

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, Any]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Any = None

        # process the path parameters
        if ontology is not None:
            _path_params["ontology"] = ontology
        if query_api_name is not None:
            _path_params["queryApiName"] = query_api_name
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        _header_params["Accept"] = self._api_client.select_header_accept(["application/json"])

        # authentication setting
        _auth_settings: List[str] = ["BearerAuth"]

        _response_types_map: Dict[str, Optional[str]] = {"200": "QueryTypeV2"}
        return self._api_client.call_api(
            method="GET",
            resource_path="/v2/ontologies/{ontology}/queryTypes/{queryApiName}",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            response_types_map=_response_types_map,
        )

    @validate_call
    @handle_unexpected
    def list_action_types(
        self,
        ontology: Annotated[
            StrictStr,
            Field(
                description="The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**. "
            ),
        ],
        page_size: Annotated[
            Optional[StrictInt],
            Field(
                description="The desired size of the page to be returned. Defaults to 500. See [page sizes](/docs/foundry/api/general/overview/paging/#page-sizes) for details. "
            ),
        ] = None,
        page_token: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]],
        ] = None,
    ) -> ListActionTypesResponseV2:
        """list_action_types

        Lists the action types for the given Ontology.  Each page may be smaller than the requested page size. However, it is guaranteed that if there are more results available, at least one result will be present in the response.  Third-party applications using this endpoint via OAuth2 must request the following operation scope: `api:read-data`.

        :param ontology: The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**.  (required)
        :type ontology: str
        :param page_size: The desired size of the page to be returned. Defaults to 500. See [page sizes](/docs/foundry/api/general/overview/paging/#page-sizes) for details.
        :type page_size: int
        :param page_token:
        :type page_token: str
        :param _request_timeout: timeout setting for this request in seconds. If
                                 one number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :return: Returns the result object.
        """  # noqa: E501

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, Any]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Any = None

        # process the path parameters
        if ontology is not None:
            _path_params["ontology"] = ontology
        # process the query parameters
        if page_size is not None:
            _query_params.append(("pageSize", page_size))

        if page_token is not None:
            _query_params.append(("pageToken", page_token))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        _header_params["Accept"] = self._api_client.select_header_accept(["application/json"])

        # authentication setting
        _auth_settings: List[str] = ["BearerAuth"]

        _response_types_map: Dict[str, Optional[str]] = {"200": "ListActionTypesResponseV2"}
        return self._api_client.call_api(
            method="GET",
            resource_path="/v2/ontologies/{ontology}/actionTypes",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            response_types_map=_response_types_map,
        )

    @validate_call
    @handle_unexpected
    def list_deployments(
        self,
        ontology: Annotated[
            StrictStr,
            Field(
                description="The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**. "
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]],
        ] = None,
    ) -> ListDeploymentsResponse:
        """list_deployments

        Fetches a list of the available model deployments within a given Ontology.

        :param ontology: The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**.  (required)
        :type ontology: str
        :param _request_timeout: timeout setting for this request in seconds. If
                                 one number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :return: Returns the result object.
        """  # noqa: E501

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, Any]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Any = None

        # process the path parameters
        if ontology is not None:
            _path_params["ontology"] = ontology
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        _header_params["Accept"] = self._api_client.select_header_accept(["application/json"])

        # authentication setting
        _auth_settings: List[str] = ["BearerAuth"]

        _response_types_map: Dict[str, Optional[str]] = {"200": "ListDeploymentsResponse"}
        return self._api_client.call_api(
            method="GET",
            resource_path="/v2/ontologies/{ontology}/models/deployments",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            response_types_map=_response_types_map,
        )

    @validate_call
    @handle_unexpected
    def list_object_types(
        self,
        ontology: Annotated[
            StrictStr,
            Field(
                description="The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**. "
            ),
        ],
        page_size: Annotated[
            Optional[StrictInt],
            Field(
                description="The desired size of the page to be returned. Defaults to 500. See [page sizes](/docs/foundry/api/general/overview/paging/#page-sizes) for details. "
            ),
        ] = None,
        page_token: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]],
        ] = None,
    ) -> ListObjectTypesV2Response:
        """list_object_types

        Lists the object types for the given Ontology.  Each page may be smaller or larger than the requested page size. However, it is guaranteed that if there are more results available, at least one result will be present in the response.  Third-party applications using this endpoint via OAuth2 must request the following operation scope: `api:read-data`.

        :param ontology: The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**.  (required)
        :type ontology: str
        :param page_size: The desired size of the page to be returned. Defaults to 500. See [page sizes](/docs/foundry/api/general/overview/paging/#page-sizes) for details.
        :type page_size: int
        :param page_token:
        :type page_token: str
        :param _request_timeout: timeout setting for this request in seconds. If
                                 one number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :return: Returns the result object.
        """  # noqa: E501

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, Any]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Any = None

        # process the path parameters
        if ontology is not None:
            _path_params["ontology"] = ontology
        # process the query parameters
        if page_size is not None:
            _query_params.append(("pageSize", page_size))

        if page_token is not None:
            _query_params.append(("pageToken", page_token))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        _header_params["Accept"] = self._api_client.select_header_accept(["application/json"])

        # authentication setting
        _auth_settings: List[str] = ["BearerAuth"]

        _response_types_map: Dict[str, Optional[str]] = {"200": "ListObjectTypesV2Response"}
        return self._api_client.call_api(
            method="GET",
            resource_path="/v2/ontologies/{ontology}/objectTypes",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            response_types_map=_response_types_map,
        )

    @validate_call
    @handle_unexpected
    def list_ontologies(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]],
        ] = None,
    ) -> ListOntologiesV2Response:
        """list_ontologies

        Lists the Ontologies visible to the current user.  Third-party applications using this endpoint via OAuth2 must request the following operation scope: `api:read-data`.

        :param _request_timeout: timeout setting for this request in seconds. If
                                 one number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :return: Returns the result object.
        """  # noqa: E501

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, Any]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Any = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        _header_params["Accept"] = self._api_client.select_header_accept(["application/json"])

        # authentication setting
        _auth_settings: List[str] = ["BearerAuth"]

        _response_types_map: Dict[str, Optional[str]] = {"200": "ListOntologiesV2Response"}
        return self._api_client.call_api(
            method="GET",
            resource_path="/v2/ontologies",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            response_types_map=_response_types_map,
        )

    @validate_call
    @handle_unexpected
    def list_outgoing_link_types(
        self,
        ontology: Annotated[
            StrictStr,
            Field(
                description="The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**. "
            ),
        ],
        object_type: Annotated[
            StrictStr,
            Field(
                description="The API name of the object type. To find the API name, use the **List object types** endpoint or check the **Ontology Manager** application. "
            ),
        ],
        page_size: Annotated[
            Optional[StrictInt], Field(description="The desired size of the page to be returned.")
        ] = None,
        page_token: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]],
        ] = None,
    ) -> ListOutgoingLinkTypesResponseV2:
        """list_outgoing_link_types

        List the outgoing links for an object type.  Third-party applications using this endpoint via OAuth2 must request the following operation scopes: `api:read-data`.

        :param ontology: The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**.  (required)
        :type ontology: str
        :param object_type: The API name of the object type. To find the API name, use the **List object types** endpoint or check the **Ontology Manager** application.  (required)
        :type object_type: str
        :param page_size: The desired size of the page to be returned.
        :type page_size: int
        :param page_token:
        :type page_token: str
        :param _request_timeout: timeout setting for this request in seconds. If
                                 one number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :return: Returns the result object.
        """  # noqa: E501

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, Any]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Any = None

        # process the path parameters
        if ontology is not None:
            _path_params["ontology"] = ontology
        if object_type is not None:
            _path_params["objectType"] = object_type
        # process the query parameters
        if page_size is not None:
            _query_params.append(("pageSize", page_size))

        if page_token is not None:
            _query_params.append(("pageToken", page_token))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        _header_params["Accept"] = self._api_client.select_header_accept(["application/json"])

        # authentication setting
        _auth_settings: List[str] = ["BearerAuth"]

        _response_types_map: Dict[str, Optional[str]] = {"200": "ListOutgoingLinkTypesResponseV2"}
        return self._api_client.call_api(
            method="GET",
            resource_path="/v2/ontologies/{ontology}/objectTypes/{objectType}/outgoingLinkTypes",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            response_types_map=_response_types_map,
        )

    @validate_call
    @handle_unexpected
    def list_query_types(
        self,
        ontology: Annotated[
            StrictStr,
            Field(
                description="The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**. "
            ),
        ],
        page_size: Annotated[
            Optional[StrictInt],
            Field(
                description="The desired size of the page to be returned. Defaults to 100. See [page sizes](/docs/foundry/api/general/overview/paging/#page-sizes) for details. "
            ),
        ] = None,
        page_token: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]],
        ] = None,
    ) -> ListQueryTypesResponseV2:
        """list_query_types

        Lists the query types for the given Ontology.          Each page may be smaller than the requested page size. However, it is guaranteed that if there are more results available, at least one result will be present in the response.          Third-party applications using this endpoint via OAuth2 must request the following operation scope: `api:read-data`.

        :param ontology: The API name of the ontology. To find the API name, use the **List ontologies** endpoint or check the **Ontology Manager**.  (required)
        :type ontology: str
        :param page_size: The desired size of the page to be returned. Defaults to 100. See [page sizes](/docs/foundry/api/general/overview/paging/#page-sizes) for details.
        :type page_size: int
        :param page_token:
        :type page_token: str
        :param _request_timeout: timeout setting for this request in seconds. If
                                 one number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :return: Returns the result object.
        """  # noqa: E501

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, Any]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Any = None

        # process the path parameters
        if ontology is not None:
            _path_params["ontology"] = ontology
        # process the query parameters
        if page_size is not None:
            _query_params.append(("pageSize", page_size))

        if page_token is not None:
            _query_params.append(("pageToken", page_token))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        _header_params["Accept"] = self._api_client.select_header_accept(["application/json"])

        # authentication setting
        _auth_settings: List[str] = ["BearerAuth"]

        _response_types_map: Dict[str, Optional[str]] = {"200": "ListQueryTypesResponseV2"}
        return self._api_client.call_api(
            method="GET",
            resource_path="/v2/ontologies/{ontology}/queryTypes",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            response_types_map=_response_types_map,
        )
